/*
   Copyright (c) 2014 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#include "includes.h"

//structure for pwm configuration mirroring
PWMCONF pwmConf;

void initPWM(PWMGEN *aV)
{
   //Enable clock for all PWM modules
   EALLOW;
   SysCtrlRegs.PCLKCR1.bit.EPWM1ENCLK = 1;  // ePWM1
   SysCtrlRegs.PCLKCR1.bit.EPWM2ENCLK = 1;  // ePWM2
   SysCtrlRegs.PCLKCR1.bit.EPWM3ENCLK = 1;  // ePWM3
   SysCtrlRegs.PCLKCR1.bit.EPWM4ENCLK = 1;  // ePWM4
   SysCtrlRegs.PCLKCR1.bit.EPWM5ENCLK = 1;  // ePWM5
   SysCtrlRegs.PCLKCR1.bit.EPWM6ENCLK = 1;  // ePWM6
   SysCtrlRegs.PCLKCR1.bit.EPWM7ENCLK = 1;  // ePWM7
   SysCtrlRegs.PCLKCR1.bit.EPWM8ENCLK = 1;  // ePWM8
   SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 1;   // Enable TBCLK
   EDIS;

   //Setup  module Sync
   EPwm1Regs.TBCTL.bit.SYNCOSEL = 0;
   EPwm2Regs.TBCTL.bit.SYNCOSEL = 0;
   EPwm3Regs.TBCTL.bit.SYNCOSEL = 0;
   EPwm4Regs.TBCTL.bit.SYNCOSEL = 0;
   EPwm5Regs.TBCTL.bit.SYNCOSEL = 0;
   EPwm6Regs.TBCTL.bit.SYNCOSEL = 0;

   //Allow each timer to be sync'ed
   EPwm1Regs.TBCTL.bit.PHSEN = 1;
   EPwm2Regs.TBCTL.bit.PHSEN = 1;
   EPwm3Regs.TBCTL.bit.PHSEN = 1;
   EPwm4Regs.TBCTL.bit.PHSEN = 1;
   EPwm5Regs.TBCTL.bit.PHSEN = 1;
   EPwm6Regs.TBCTL.bit.PHSEN = 1;

   //Init Timer-Base Period Register for EPWM1-EPWM3
   EPwm1Regs.TBPRD = aV->PeriodMax;
   EPwm2Regs.TBPRD = aV->PeriodMax;
   EPwm3Regs.TBPRD = aV->PeriodMax;

   //Init Timer-Base Phase Register for EPWM1-EPWM3
   EPwm1Regs.TBPHS.half.TBPHS = 0;
   EPwm2Regs.TBPHS.half.TBPHS = 0;
   EPwm3Regs.TBPHS.half.TBPHS = 0;

   //Init Timer-Base Control Register for EPWM1-EPWM3
   EPwm1Regs.TBCTL.all = BLDCPWM_INIT_STATE;
   EPwm2Regs.TBCTL.all = BLDCPWM_INIT_STATE;
   EPwm3Regs.TBCTL.all = BLDCPWM_INIT_STATE;

   // Init Compare Control Register for EPWM1-EPWM3
   EPwm1Regs.CMPCTL.all = BLDCPWM_CMPCTL_INIT_STATE;
   EPwm2Regs.CMPCTL.all = BLDCPWM_CMPCTL_INIT_STATE;
   EPwm3Regs.CMPCTL.all = BLDCPWM_CMPCTL_INIT_STATE;

   // Init Action Qualifier Output A Register for EPWM1-EPWM3
   EPwm1Regs.AQCTLA.all = BLDCPWM_AQCTLA_INIT_STATE;
   EPwm2Regs.AQCTLA.all = BLDCPWM_AQCTLA_INIT_STATE;
   EPwm3Regs.AQCTLA.all = BLDCPWM_AQCTLA_INIT_STATE;

   //Init Dead-Band Generator Control Register for EPWM1-EPWM3
   EPwm1Regs.DBCTL.all = BLDCPWM_DBCTL_INIT_STATE;
   EPwm2Regs.DBCTL.all = BLDCPWM_DBCTL_INIT_STATE;
   EPwm3Regs.DBCTL.all = BLDCPWM_DBCTL_INIT_STATE;

   //Init PWM Chopper Control Register for EPWM1-EPWM3
   EPwm1Regs.PCCTL.all = BLDCPWM_PCCTL_INIT_STATE;
   EPwm2Regs.PCCTL.all = BLDCPWM_PCCTL_INIT_STATE;
   EPwm3Regs.PCCTL.all = BLDCPWM_PCCTL_INIT_STATE;

   EALLOW;

   //Initialize protections (TZ2 & TZ3) for epwm1-epwm3
   EPwm1Regs.TZSEL.bit.OSHT2 = 1;
   EPwm1Regs.TZSEL.bit.OSHT3 = 1;
   EPwm1Regs.TZCTL.bit.TZB = TZ_FORCE_LO;
   //EPwm1Regs.TZEINT.bit.OST = 1; // Enable TZ interrupt
   EPwm2Regs.TZSEL.bit.OSHT2 = 1;
   EPwm2Regs.TZSEL.bit.OSHT3 = 1;
   EPwm2Regs.TZCTL.bit.TZB = TZ_FORCE_LO;
   //EPwm1Regs.TZEINT.bit.OST = 1; // Enable TZ interrupt
   EPwm3Regs.TZSEL.bit.OSHT2 = 1;
   EPwm3Regs.TZSEL.bit.OSHT3 = 1;
   EPwm3Regs.TZCTL.bit.TZB = TZ_FORCE_LO; // reset driver
   //  EPwm1Regs.TZEINT.bit.OST = 1; // Enable TZ interrupt

   //Setting six EPWM as primary output pins
   GpioCtrlRegs.GPAMUX1.bit.GPIO0 = 1; // EPWM1A pin
   GpioCtrlRegs.GPAMUX1.bit.GPIO1 = 1; // EPWM1B pin
   GpioCtrlRegs.GPAMUX1.bit.GPIO2 = 1; // EPWM2A pin
   GpioCtrlRegs.GPAMUX1.bit.GPIO3 = 1; // EPWM2B pin
   GpioCtrlRegs.GPAMUX1.bit.GPIO4 = 1; // EPWM3A pin
   GpioCtrlRegs.GPAMUX1.bit.GPIO5 = 1; // EPWM3B pin

   EDIS;

   //Configure Event Trigger module for current measurement
   EPwm1Regs.ETSEL.bit.SOCAEN = 1;  // Enable SOCA
   EPwm1Regs.ETSEL.bit.SOCASEL = 6; // Enable CMPB event for SOCA
   EPwm1Regs.ETPS.bit.SOCAPRD = 1;  // Generate SOCA on the 1st event
   EPwm1Regs.ETCLR.bit.SOCA = 1;    // Clear SOCA flag
}

/*
* Update PWM configuration mirror with actual settings
*/
void updateConfigMirror()
{
   //ePWM1
   pwmConf.PWM1CMPA = EPwm1Regs.CMPA.half.CMPA;
   pwmConf.PWM1CMPB = EPwm1Regs.CMPB;
   pwmConf.PWM1AQCSFRC = EPwm1Regs.AQCSFRC.all;

   //ePWM2
   pwmConf.PWM2CMPA= EPwm2Regs.CMPA.half.CMPA;
   pwmConf.PWM2AQCSFRC = EPwm2Regs.AQCSFRC.all;

   //ePWM3
   pwmConf.PWM3CMPA = EPwm3Regs.CMPA.half.CMPA;
   pwmConf.PWM3AQCSFRC = EPwm3Regs.AQCSFRC.all;
}

/*
 * Update the configuration of the used pwm modules
 */
void updatePWM(PWMGEN *aV)
{
   int32 Tmp, Tmp2;
   int16 Period, GPR0_BLDC_PWM;
   int16 CMPB_SMPL_POINT;

   //Convert "Period" (Q15) modulation function to Q0
   Tmp = (int32) aV->PeriodMax * (int32) aV->MfuncPeriod; //Q15 = Q0xQ15
   Period = (int16) (Tmp >> 15);                        //Q15 -> Q0 (Period)

   // Check PwmActive setting
   if (aV->PwmActive == 1)
   {
      //PWM active high
      GPR0_BLDC_PWM = 0x7FFF - aV->DutyFunc;
      CMPB_SMPL_POINT = 0x7FFF - (aV->DutyFunc >> 1);    //Sample in center of PWM pulse using CMPB
   }
   else if (aV->PwmActive == 0)
   {
      GPR0_BLDC_PWM = aV->DutyFunc;
      CMPB_SMPL_POINT = aV->DutyFunc >> 1;               //Sample in center of PWM pulse using CMPB
   }

   //Convert "DutyFunc" or "GPR0_BLDC_PWM" (Q15) duty modulation function to Q0
   Tmp = (int32) Period * (int32) GPR0_BLDC_PWM;        //Q15 = Q0xQ15
   Tmp2 = (int32) Period * (int32) CMPB_SMPL_POINT;     //Q15 = Q0xQ15
   EPwm1Regs.CMPB = (int16) (Tmp2 >> 15);               //Sample in center of PWM pulse using CMPB

   //State s1: current from phase A->B, C de-energized
   if (aV->CmtnPointer == 0)
   {
      EPwm1Regs.AQCSFRC.bit.CSFB = 0;                    //Forcing disabled on output B of EPWM1
      EPwm1Regs.AQCTLB.bit.CAU = 2;                      //Set high when CTR = CMPA on UP-count
      EPwm1Regs.AQCTLB.bit.ZRO = 1;                      //Set low when CTR = Zero
      EPwm1Regs.CMPA.half.CMPA = (int16) (Tmp >> 15);    //PWM signal on output B of EPWM1 (Q15 -> Q0)

      EPwm1Regs.AQCSFRC.bit.CSFA = 2;                    //Forcing a continuous High on output A of EPWM1

      EPwm2Regs.AQCSFRC.bit.CSFA = 1;                    //Forcing a continuous Low on output A of EPWM2
      EPwm2Regs.AQCSFRC.bit.CSFB = 2;                    //Forcing a continuous High on output B of EPWM2

      EPwm3Regs.AQCSFRC.bit.CSFA = 1;                    //Forcing a continuous Low on output A of EPWM3
      EPwm3Regs.AQCSFRC.bit.CSFB = 1;                    //Forcing a continuous Low on output B of EPWM3
   }

   //State s2: current from phase A->C, B de-energized
   else if (aV->CmtnPointer == 1)
   {
      EPwm1Regs.AQCSFRC.bit.CSFB = 0;                    //Forcing disabled on output B of EPWM1
      EPwm1Regs.AQCTLB.bit.CAU = 2;                      //Set high when CTR = CMPA on UP-count
      EPwm1Regs.AQCTLB.bit.ZRO = 1;                      //Set low when CTR = Zero
      EPwm1Regs.CMPA.half.CMPA = (int16) (Tmp >> 15);    //PWM signal on output B of EPWM1 (Q15 -> Q0)
      EPwm1Regs.AQCSFRC.bit.CSFA = 2;                    //Forcing a continuous High on output A of EPWM1

      EPwm2Regs.AQCSFRC.bit.CSFA = 1;                    //Forcing a continuous Low on output A of EPWM2
      EPwm2Regs.AQCSFRC.bit.CSFB = 1;                    //Forcing a continuous Low on output B of EPWM2

      EPwm3Regs.AQCSFRC.bit.CSFA = 1;                    //Forcing a continuous Low on output A of EPWM3
      EPwm3Regs.AQCSFRC.bit.CSFB = 2;                    //Forcing a continuous High on output B of EPWM3
   }

   //State s3: current from phase B->C, A de-energized
   else if (aV->CmtnPointer == 2)
   {
      EPwm1Regs.AQCSFRC.bit.CSFA = 1;                    //Forcing a continuous Low on output A of EPWM1
      EPwm1Regs.AQCSFRC.bit.CSFB = 1;                    //Forcing a continuous Low on output B of EPWM1

      EPwm2Regs.AQCSFRC.bit.CSFB = 0;                    //Forcing disabled on output B of EPWM2
      EPwm2Regs.AQCTLB.bit.CAU = 2;                      //Set high when CTR = CMPA on UP-count
      EPwm2Regs.AQCTLB.bit.ZRO = 1;                      //Set low when CTR = Zero
      EPwm2Regs.CMPA.half.CMPA = (int16) (Tmp >> 15);    //PWM signal on output B of EPWM2 (Q15 -> Q0)
      EPwm2Regs.AQCSFRC.bit.CSFA = 2;                    //Forcing a continuous High on output A of EPWM2

      EPwm3Regs.AQCSFRC.bit.CSFA = 1;                    //Forcing a continuous Low on output A of EPWM3
      EPwm3Regs.AQCSFRC.bit.CSFB = 2;                    //Forcing a continuous High on output B of EPWM3
   }

   //State s4: current from phase B->A, C de-energized
   else if (aV->CmtnPointer == 3)
   {
      EPwm1Regs.AQCSFRC.bit.CSFA = 1;                    //Forcing a continuous Low on output A of EPWM1
      EPwm1Regs.AQCSFRC.bit.CSFB = 2;                    //Forcing a continuous High on output B of EPWM1

      EPwm2Regs.AQCSFRC.bit.CSFB = 0;                    //Forcing disabled on output B of EPWM2
      EPwm2Regs.AQCTLB.bit.CAU = 2;                      //Set high when CTR = CMPA on UP-count
      EPwm2Regs.AQCTLB.bit.ZRO = 1;                      //Set low when CTR = Zero
      EPwm2Regs.CMPA.half.CMPA = (int16) (Tmp >> 15);    //PWM signal on output B of EPWM2 (Q15 -> Q0)
      EPwm2Regs.AQCSFRC.bit.CSFA = 2;                    //Forcing a continuous High on output A of EPWM2

      EPwm3Regs.AQCSFRC.bit.CSFA = 1;                    //Forcing a continuous Low on output A of EPWM3
      EPwm3Regs.AQCSFRC.bit.CSFB = 1;                    //Forcing a continuous Low on output B of EPWM3
   }

   //State s5: current from phase C->A, B de-energized
   else if (aV->CmtnPointer == 4)
   {
      EPwm1Regs.AQCSFRC.bit.CSFA = 1;                    //Forcing a continuous Low on output A of EPWM1
      EPwm1Regs.AQCSFRC.bit.CSFB = 2;                    //Forcing a continuous High on output B of EPWM1

      EPwm2Regs.AQCSFRC.bit.CSFA = 1;                    //Forcing a continuous Low on output A of EPWM2
      EPwm2Regs.AQCSFRC.bit.CSFB = 1;                    //Forcing a continuous Low on output B of EPWM2

      EPwm3Regs.AQCSFRC.bit.CSFB = 0;                    //Forcing disabled on output B of EPWM3
      EPwm3Regs.AQCTLB.bit.CAU = 2;                      //Set high when CTR = CMPA on UP-count
      EPwm3Regs.AQCTLB.bit.ZRO = 1;                      //Set low when CTR = Zero
      EPwm3Regs.CMPA.half.CMPA = (int16) (Tmp >> 15);    //PWM signal on output B of EPWM3 (Q15 -> Q0)
      EPwm3Regs.AQCSFRC.bit.CSFA = 2;                    //Forcing a continuous High on output A of EPWM3
   }

   //State s6: current from phase C->B, A de-energized
   else if (aV->CmtnPointer == 5)
   {
      EPwm1Regs.AQCSFRC.bit.CSFA = 1;                    //Forcing a continuous Low on output A of EPWM1
      EPwm1Regs.AQCSFRC.bit.CSFB = 1;                    //Forcing a continuous Low on output B of EPWM1

      EPwm2Regs.AQCSFRC.bit.CSFA = 1;                    //Forcing a continuous Low on output A of EPWM2
      EPwm2Regs.AQCSFRC.bit.CSFB = 2;                    //Forcing a continuous High on output B of EPWM2

      EPwm3Regs.AQCSFRC.bit.CSFB = 0;                    //Forcing disabled on output B of EPWM3
      EPwm3Regs.AQCTLB.bit.CAU = 2;                      //Set high when CTR = CMPA on UP-count
      EPwm3Regs.AQCTLB.bit.ZRO = 1;                      //Set low when CTR = Zero
      EPwm3Regs.CMPA.half.CMPA = (int16) (Tmp >> 15);    //PWM signal on output B of EPWM3 (Q15 -> Q0)
      EPwm3Regs.AQCSFRC.bit.CSFA = 2;                    //Forcing a continuous High on output A of EPWM3
   }
   //update configuration mirror
   updateConfigMirror();
}

