/*
   Copyright (c) 2014 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#include "includes.h"

#ifdef USE_SCI_LINK

void SCIInit (Uint16 mode,
		Uint32 baudrate,
		Uint32 clk
){
	int16 brr;

	EALLOW;

	GpioCtrlRegs.GPAQSEL2.bit.GPIO28 = 3;
	GpioCtrlRegs.GPAMUX2.bit.GPIO28 = 1;
	GpioCtrlRegs.GPAMUX2.bit.GPIO29 = 1;

	SysCtrlRegs.PCLKCR0.bit.SCIAENCLK=1;

	SciaRegs.SCICTL1.all = 0;
	SciaRegs.SCICCR.all = mode;

	brr = (Uint16)(clk /8l /baudrate) - 1;
	SciaRegs.SCIHBAUD = 0xFF & (brr>>8);
	SciaRegs.SCILBAUD = 0xFF & brr;

#ifdef USE_SCI_FIFO
	SciaRegs.SCIFFTX.all=0xC000;
	SciaRegs.SCIFFRX.all=0x0000;
	SciaRegs.SCIFFCT.all=0x00;
#endif

	SciaRegs.SCICTL1.all = 0x0033;

#ifdef USE_SCI_FIFO
	SciaRegs.SCIFFTX.bit.TXFIFOXRESET=1;
	SciaRegs.SCIFFRX.bit.RXFIFORESET=1;
#endif

	EDIS;
}


int16 SCIGetChar(void){
#ifdef USE_SCI_FIFO
	if(SciaRegs.SCIFFRX.bit.RXFFST == 0){
#else
	if(SciaRegs.SCIRXST.bit.RXRDY == 0){
#endif
    	// nothing there...
        return(-1);
    }else{
		return((int16)SciaRegs.SCIRXBUF.all);
    }
}

void SCIPutChar(Uint8 c){	
    while(SciaRegs.SCICTL2.bit.TXRDY == 0){
		continue;
    }
	SciaRegs.SCITXBUF = (Uint16)c;
}

void SCIWriteString(const char *s){
	Uint16 sc;

	sc = (Uint8)(*s);
	while(sc != 0){
		SCIPutChar((Uint8)sc);
		s++;
		sc = (Uint8)(*s);
	}
}

#ifndef USE_SCI_FIFO
#error SCIPoll() implementation assumes use of FIFO!
#endif
void SCIPoll()
{
	while(SciaRegs.SCIFFRX.bit.RXFFST != 0)
	{
		// assuming that there will be a "break" when FIFO is empty
		PIL_RA_serialIn((int16)SciaRegs.SCIRXBUF.all);
	}

	int16_t ch;
	if(SciaRegs.SCICTL2.bit.TXRDY == 1){
		if(PIL_RA_serialOut(&ch))
		{
			SciaRegs.SCITXBUF = ch;
		}
	}
}

#endif
