% electrical
Vbat = 100; % battery voltage
Cbus = 2.5e-3; % bus capacitance
Rbus = 0.001; % bus cap ESR

% mechanical (shaft) speed
Wm = 1.0*2*pi*5000/60; % radians per second

% PWM and sampling
fpwm = 10000;
fs = fpwm;
Tpwm = 1/fpwm;
Tdt = 0.01*Tpwm; % deadtime
Ts = 1/fs;

Tcs = 1/(2*pi*100000); % current sensor bandwidth

% "R" Parameters
Mot.Rs = 0.00235;
Mot.Ld = 0.000038; 
Mot.Lq = 0.000096;
Mot.Lss = 10e-6;
Mot.P = 2*3;
Mot.Psir  = 0.04;
Mot.Id0 = -200.0;
Mot.Iq0 = 0;

Mot.thm0 = 0;
% assumes initial position is zero!!!
Mot.is0 = [Mot.Id0 (-0.5*Mot.Id0 + sqrt(3)/2* Mot.Iq0)];

% equivalent small time-constant (used for current regulator tuning)
Tcalc = 0.5/fs;
Ta = 0.5/fpwm;
Tsh = 0.5/fs;
Tp = Tcalc + Ta + Tsh + Tcs;

% current regulator tuning
Ra = 0.0;
R = Mot.Rs+Ra;

Tii = 2/R*Tp;
Tni_D = Mot.Ld/R;
Tni_Q = Mot.Lq/R;

Ki = 1/Tii;
Kpd = Tni_D/Tii;
Kpq = Tni_Q/Tii;

% current regulator
IReg.Ra = Ra;

IReg.Tcalc = Tcalc;
IReg.Ts = Ts;
IReg.Tn_D = Tni_D;
IReg.Tn_Q = Tni_Q;
IReg.Ti = Tii;
IReg.Kp = [Kpd Kpq];
IReg.Ki = [Ki Ki];

IReg.V0 = [0 (Wm*Mot.P/2*(Mot.Psir+Mot.Ld*Mot.Id0))];