% Plant parameters
% =======================================================================

%% Inertias and masses in system
P.Jm = 0.001; % motor
P.Jc = 0.0003; % coupling
P.Js = 0.0005; % screw
P.msp = 1000; % mass of slide and part

%% Compliance
P.K = 100000; % spring constant (should be defined in pu for consistency)

%% ball screw lead (displacement per revolution)
P.dpr = 0.012; 

%% gear backlash
P.bl = 2*pi/2000;

%% Hard stop for slide
P.hs = 0.07;

%% Total (equivalent) inertia
P.Jtot = P.Jm + P.Jc + P.Js + P.msp * (P.dpr / (2*pi))^2;

% Position Controller
% =======================================================================

%% Per-unit references
R1.T_ref = 6; % nominal torque of electric machine
R1.W_ref = 4000/30*pi; % nominal speed of electric machine
R1.Theta_ref = 0.2*2*pi; % arbitrary
R1.r_ref = P.dpr / (2*pi);
R1.d_ref = R1.Theta_ref * R1.r_ref; % reference for displacement
R1.v_ref = R1.W_ref * R1.r_ref; % reference for translational speed
R1.P_ref = R1.T_ref * R1.W_ref; % power reference

%% Plant parameters (in pu)
R1.Tm = P.Jtot * R1.W_ref/R1.T_ref; % mechanical time-constant 
	
%% Equivalent delays limiting achievable bandwidth
R1.Ts = 1.0e-3;
R1.Tem = 0.033e-3;
R1.Tr = 0.167e-3;
R1.Tmes = R1.Ts/2;

R1.TpEp = R1.Ts/2+R1.Tem+R1.Tr;
R1.TpE = R1.TpEp + R1.Tmes;


% Electric drive
% =======================================================================
D.Vbat = 400; % battery voltage
D.Cbus = 2.5e-3; % bus capacitance
D.Rbus = 0.001; % bus cap ESR

% PWM and sampling
D.fpwm = 10000;
D.fs = D.fpwm;
D.Tpwm = 1/D.fpwm;
D.Tdt = 0.01*D.Tpwm; % dead time;
D.Ts = 1/D.fs;
D.Tcs = 1/(2*pi*100000); % current sensor bandwidth

%% Motor Parameters (low-inertia servo BSM90N-175A)
M.Rs = 1.24/2;
M.Ld = 0.00414; 
M.Lq = 0.00414;
M.Lss = (M.Ld + M.Lq)/50;
M.P = 2*4;
M.Psir  = 0.101;
%% Initial conditions assuming initial position = 0 and Y connection
M.wm0 = 0;
M.thm0 = 0;
M.Id0 = 0;
M.Iq0 = 0;
M.Is0 = [0 0];

%% equivalent small time constant (used for current regulator tuning)
Ri.Ts = D.Ts;
Ri.Tcalc = 0.5/D.fs;
Ri.Ta = 0.5/D.fpwm;
Ri.Tsh = 0.5/D.fs;
Ri.Tp = Ri.Tcalc + Ri.Ta + Ri.Tsh + D.Tcs;

%% current regulator tuning
Ri.Ra = 0.0;
Ri.R = M.Rs + Ri.Ra;

Ri.Tii = 2/Ri.R*Ri.Tp;
Ri.Tni_D = M.Ld/Ri.R;
Ri.Tni_Q = M.Lq/Ri.R;

%%% analog gains
Ria.Ki_D = 1/Ri.Tii;
Ria.Ki_Q = 1/Ri.Tii;
Ria.Kp_D = Ri.Tni_D/Ri.Tii;
Ria.Kp_Q = Ri.Tni_Q/Ri.Tii;
Ri.Kp = [Ria.Kp_D Ria.Kp_Q];
Ri.Ki = [Ria.Ki_D Ria.Ki_Q];

%%% initial conditions
Ri.Vd0 = M.Rs * M.Iq0;
Ri.Vq0 = M.Rs * M.Id0 + M.wm0*M.P/2*(M.Psir+M.Ld*M.Id0);
Ri.V0 = [Ri.Vd0 Ri.Vq0];