%% Vehicle Parameters
m_v=560; % [kg] - Wheel mass
g=9.81; % [m/s^2] - Acceleration due to gravity
F_v=m_v*g; % [N] - Weight
v_init=0; %[m/s] Initial vehicle velocity
l_base=2.44; % [m] - Length of vehicle wheelbase
h=0.488; % [m] - Height of vehicle center of gravity (CG)
l_f=1.0492; % [m] - Distance of CG from front axle
l_r=l_base-l_f; % [m] - Distance of CG from rear axle

%% Motor and gear parameters
J_motor=.014; % [kgm^2] - Motor inertia
w_m_init=0; % [rad/s] - Initial motor speed
g_ratio=5; % [1] - Gear ratio
shaft_Ks = 120; % [Nm/rad] - Rotor shaft stiffness
shaft_Ds = 5; % [Nms/rad] - Rotor damping coefficient
gear_bl = 0;% [rads] - Gear backlash

%% Wheel Parameters 
m_w = 11.38; % [kg] - Wheel mass
r_w = 0.321;% [m] - Wheel radius
J_w = m_w*r_w^2; %[kgm^2] - Wheel inertia
w_w_init=w_m_init/g_ratio; % [rad/s] - Initial wheel speed
u_rr=0.01; % [1] - Rolling resistance
w_fric=u_rr*F_v/2*r_w; % [Nm] - Breakaway & Coulomb wheel friction torque

% Driving and Braking Torque
g_brk = 0.4;  % [1] - Braking torque as a percentage of vehicle weight
T_brk_rear = 0.4*g_brk*F_v*r_w; % [Nm] - 40 percent on rear wheels
T_brk_front = 0.6*g_brk*F_v*r_w; % [Nm] - 60 percent on front wheels
T_drive=0.2*m_v*g/g_ratio*r_w; % [Nm] - Driving torque

%% Magic Formula Coefficients
Dx=3000; % [N]
Cx=1.6; % [1]
Bx=12.5; % [1]
dx=Dx/(600*g); % [1] - linearized coefficient

%% Slip Calculation coefficients
lambda_init=0; % [1] - Initial slip
sigma_min=0.02; % [m] - Minimum tire relaxation length
sigma_k0=0.2; % [m] - Low speed relaxation length filter coefficient
k_V_low0=770; % [Ns/m] - Low speed filtering coefficient

%% Delay
R1_Tmes = 0.5e-3; % [s] - Delay in the change of vehicle pitch angle

%% Road Load
RdLoad_BFF = 10; % [N] - Breakaway road load force
RdLoad_VFC = 50; % [Ns/m] - Viscous road load coefficient

%% Drive Cycles:
% Mechanical Braking
Time_mech=[0, 		100, 			100.1, 	200, 200.1, 300, 		310,  				450, 	450.1, 482];
Torque_mech=[0, 0.6*T_drive, T_drive, T_drive, 0, 	0, 		-.5*T_drive, -.5*T_drive, 0, 	0];

% Regenerative Braking (electrical+mechanical braking):
Time_regen=[0, 		100, 			100.1, 	200, 		200.1, 		206, 	206.1, 300, 		310,				450,		450.1,		453,		453.1, 482];
Torque_regen=[0, 0.6*T_drive, T_drive, T_drive, -T_drive, -T_drive, 0, 		0, -.5*T_drive, -.5*T_drive, T_drive, 	T_drive,		0,	0];